package config

import (
	"errors"
	"fmt"
	"io/fs"
	"io/ioutil"
	"os"
	"path"
	"path/filepath"
	"strings"

	"github.com/joho/godotenv"
	"github.com/sirupsen/logrus"
)

var ABRA_DIR = os.ExpandEnv("$HOME/.abra")
var ABRA_SERVER_FOLDER = path.Join(ABRA_DIR, "servers")
var APPS_JSON = path.Join(ABRA_DIR, "apps.json")

// Type aliases to make code hints easier to understand
type AppEnv = map[string]string
type AppName = string

type App struct {
	Name   AppName
	Type   string
	Domain string
	Env    AppEnv
}
type AppFile struct {
	Path   string
	Server string
}

type AppFiles = map[AppName]AppFile

func LoadAppFiles(servers ...string) (AppFiles, error) {
	appFiles := make(AppFiles)
	if len(servers) == 1 && servers[0] == "" {
		// Empty servers flag, one string will always be passed
		var err error
		servers, err = getAllFoldersInDirectory(ABRA_SERVER_FOLDER)
		if err != nil {
			return nil, err
		}
	}
	for _, server := range servers {
		serverDir := path.Join(ABRA_SERVER_FOLDER, server)
		files, err := getAllFilesInDirectory(serverDir)
		if err != nil {
			return nil, err
		}
		for _, file := range files {
			appName := strings.TrimSuffix(file.Name(), ".env")
			appFilePath := path.Join(ABRA_SERVER_FOLDER, server, file.Name())
			appFiles[appName] = AppFile{
				Path:   appFilePath,
				Server: server,
			}
		}
	}
	return appFiles, nil
}

// GetApp loads an apps settings, reading it from file, in preparation to use it
//
// ONLY use when ready to use the env file to keep IO down and
// because this exits with code 1 if the file cannot be found or is malformed
func GetApp(apps AppFiles, name AppName) (App, error) {
	appFile, exists := apps[name]
	if !exists {
		return App{}, fmt.Errorf("cannot find app file with name '%s'", name)
	}
	app, err := readAppFile(appFile, name)
	if err != nil {
		return App{}, err
	}
	return app, nil
}

// TODO: maybe better names than read and get

func readAppFile(appFile AppFile, name AppName) (App, error) {
	env, err := readEnv(appFile.Path)
	if err != nil {
		return App{}, fmt.Errorf("env file for '%s' couldn't be read: %s", name, err.Error())
	}
	app, err := makeApp(env, name)
	if err != nil {
		return App{}, fmt.Errorf("env file for '%s' has issues: %s", name, err.Error())
	}
	return app, nil
}

func readEnv(filePath string) (AppEnv, error) {
	var envFile AppEnv
	envFile, err := godotenv.Read(filePath)
	if err != nil {
		return nil, err
	}
	return envFile, nil
}

func makeApp(env AppEnv, name string) (App, error) {
	// Checking for type as it is required - apps wont work without it
	domain := env["DOMAIN"]
	apptype, ok := env["TYPE"]
	if !ok {
		return App{}, errors.New("missing TYPE variable")
	}
	return App{
		Name:   name,
		Domain: domain,
		Type:   apptype,
		Env:    env,
	}, nil
}

func ReadServerNames() ([]string, error) {
	serverNames, err := getAllFoldersInDirectory(ABRA_SERVER_FOLDER)
	if err != nil {
		return nil, err
	}
	return serverNames, nil
}

// getAllFilesInDirectory returns filenames of all files in directory
func getAllFilesInDirectory(directory string) ([]fs.FileInfo, error) {
	var realFiles []fs.FileInfo
	files, err := ioutil.ReadDir(directory)
	if err != nil {
		return nil, err
	}
	for _, file := range files {
		// Follow any symlinks
		filePath := path.Join(directory, file.Name())
		realPath, err := filepath.EvalSymlinks(filePath)
		if err != nil {
			logrus.Warningf("broken symlink in your abra config folders: '%s'", filePath)
		} else {
			realFile, err := os.Stat(realPath)
			if err != nil {
				return nil, err
			}
			if !realFile.IsDir() {
				realFiles = append(realFiles, file)
			}
		}

	}
	return realFiles, nil
}

// getAllFoldersInDirectory returns both folder and symlink paths
func getAllFoldersInDirectory(directory string) ([]string, error) {
	var folders []string
	files, err := ioutil.ReadDir(directory)
	if err != nil {
		return nil, err
	}
	if len(files) == 0 {
		return nil, fmt.Errorf("directory is empty: '%s'", directory)
	}
	for _, file := range files {
		// Check if file is directory or symlink
		if file.IsDir() || file.Mode()&fs.ModeSymlink != 0 {
			filePath := path.Join(directory, file.Name())
			realDir, err := filepath.EvalSymlinks(filePath)
			if err != nil {
				logrus.Warningf("broken symlink in your abra config folders: '%s'", filePath)
			} else if stat, err := os.Stat(realDir); err == nil && stat.IsDir() {
				// path is a directory
				folders = append(folders, file.Name())
			}
		}
	}
	return folders, nil
}
